#ifndef __BOOST__INDENT
#define __BOOST__INDENT
#  include <iostream>

   namespace boost
   {
      template< class OutputFileType >
      class indentor: public OutputFileType
      {
         private: // indentation
            struct
            {
               long                    level;
               char                    fill;     // templatize to CharT?
               char                    length;
            }                          m_indent;
         public:
                   indentor &                    indent();
            inline indentor &                    beginIndent() throw();
            inline indentor &                    endIndent()   throw();
            inline void                          set_indent( char = 3, char = ' ' ) throw();
         public:
            class block
            {
               private:
                  indentor *           i;
               public:
                  inline     block( indentor & in ) throw(): i( &in )
                  {
                     i -> beginIndent();
                  }
                  inline    ~block() throw()
                  {
                     i -> endIndent();
                  }
            };
         private:
            inline indentor &     operator=( const indentor & ) throw(); // not implemented
            inline           indentor( const indentor & ) throw();       // not implemented
         public:
            inline           indentor() throw();
      };

      template< class OutputFileType >
      indentor< OutputFileType > & indentor< OutputFileType >::indent()
      {
         if( m_indent.level > 0 )
         {
            char                       buff[ 300 ];
            std::memset( buff, m_indent.fill, 300 );
            *( buff + 299 ) = '\0';

            unsigned long              len = m_indent.length * m_indent.level;
            while( len > 0 )
            {
               if( len > 300 )         len -= 299;
               else
               {
                  *( buff + len ) = '\0';
                  len = 0;
               }
               *this << buff; // operator<<( const char * ) must be implemented in OutputFileType
            }
         }
         return( *this );
      }
      template< class OutputFileType >
      indentor< OutputFileType > & indentor< OutputFileType >::beginIndent() throw()
      {
         ++m_indent.level;
         return( *this );
      }
      template< class OutputFileType >
      indentor< OutputFileType > & indentor< OutputFileType >::endIndent() throw()
      {
         --m_indent.level;
         return( *this );
      }

      template< class OutputFileType >
      void indentor< OutputFileType >::set_indent( char l, char f ) throw()
      {
         m_indent.length = l;
         m_indent.fill   = f;
      }

      /*
      // [review]: MS bug (VC7 - has VC7.1 fixed this?)
      // >> cannot handle a nested class inside a template
      
      template< class OutputFileType >
      indentor< OutputFileType >::block::block( indentor & in ) throw(): i( in )
      {
         i.beginIndent();
      }
      template< class OutputFileType >
      indentor< OutputFileType >::block::~block( indentor & in ) throw(): i( in )
      {
         i.beginIndent();
      }
      //*/

      template< class OutputFileType >
      indentor< OutputFileType >::indentor() throw()
      {
         m_indent.level  = 0;
         m_indent.fill   = ' '; // static_cast< CharT >( ' ' );
         m_indent.length = 3;
      }
   }
#endif
